/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <fcntl.h>

#define PORT 65432
#define BUFSIZE 128
#define BACKLOG 1024
#define MAX_SEQ_ID 10
#define ACK "ACK"
#define NAK "NAK"

#define DISCARD_ID 6
#define NAK_ID 8

int main(void)
{
  int s_sock, c_sock;
  struct sockaddr_in server, other;
  socklen_t other_size;
  char buf[BUFSIZE], buf2[BUFSIZE];
  bool discard_flag, nak_flag;
  int i, n;

  if ((s_sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror("socket");
    exit(1);
  }

  memset(&server, 0, sizeof(server));
  memset(&other, 0, sizeof(other));
  server.sin_family = AF_INET;
  server.sin_port = htons(PORT);
  server.sin_addr.s_addr = INADDR_ANY;

  if (bind(s_sock, (struct sockaddr *) &server, sizeof(server)) == -1) {
    perror("bind");
    goto end;
  }

  if (listen(s_sock, BACKLOG) == -1) {
    perror("listen");
    goto end;
  }

  other_size = sizeof(other);

  if ((c_sock = accept(s_sock, (struct sockaddr *) &other, &other_size)) == -1) {
    perror("accept");
    goto end;
  }

  discard_flag = nak_flag = true;

  for (i = 0; i <= MAX_SEQ_ID; i++) {
    memset(buf, 0, sizeof(buf));

    if ((n = read(c_sock, buf, sizeof(buf))) == -1) {
      perror("read");
      goto end2;
    }

    buf[n] = '\0';

    sprintf(buf2, "%d", i);

    if (i == DISCARD_ID && discard_flag) {
      discard_flag = false;
      printf("\n[Discard packet]\n");
      i--;
      continue;
    }

    printf("Message from client: %s\n", buf);

    if (i == NAK_ID && nak_flag) {
      nak_flag = false;
      i--;

      if (write(c_sock, NAK, sizeof(NAK)) == -1) {
        perror("write");
        goto end2;
      }
    } else {
      if (write(c_sock, ACK, sizeof(ACK)) == -1) {
        perror("write");
        goto end2;
      }
    }
  }

end2:

  if (close(c_sock) == -1) {
    perror("close");
    exit(2);
  }

end:

  if (close(s_sock) == -1) {
    perror("close");
    exit(3);
  }

  return 0;
}
